//
//  NSDataReader.m
//  SQLKit
//
//  Created by Richard Penwell on 2/3/10.
//  Copyright 2010 Richard Penwell. All rights reserved.
//

#import "NSDataReader.h"


@implementation NSDataReader

@synthesize position;

+ (NSDataReader*)readerWithData:(NSData*)theData
{
	return [[[NSDataReader alloc] initWithData:theData] autorelease];
}

- (NSDataReader*)initWithData:(NSData*)theData
{
	if (self = [super init])
	{
		position = 0;
		data = [theData retain];
	}
	
	return self;
}

- (UInt8)readByte
{
	UInt8 theByte;
	[data getBytes:&theByte range:(NSRange){position, 1}];
	position++;
	return theByte;
}

- (UInt16)readUShort
{
	UInt16 theUShort;
	[data getBytes:(void*)&theUShort range:(NSRange){position, 2}];
	position += 2;
	return theUShort;
}

- (UInt32)readUInt
{
	UInt32 theUInt;
	[data getBytes:(void*)&theUInt range:(NSRange){position, 4}];
	position += 4;
	return theUInt;
}

- (UInt16)readUShortWithByteOrder:(CFByteOrder)order
{
	if (order == CFByteOrderBigEndian)
		return CFSwapInt16BigToHost([self readUShort]);
	else 
		return CFSwapInt16LittleToHost([self readUShort]);
}

- (UInt32)readUIntWithByteOrder:(CFByteOrder)order
{
	if (order == CFByteOrderBigEndian)
		return CFSwapInt32BigToHost([self readUInt]);
	else 
		return CFSwapInt32LittleToHost([self readUInt]);
}

- (NSData*)readDataWithLength:(NSUInteger)length
{
	return [data subdataWithRange:(NSRange){position, length}];
}

- (NSData*)remainder
{
	return [data subdataWithRange:NSMakeRange(position, ([data length] - position))];
}

- (BOOL)hasMoreData
{
	return position < [data length];
}

@end
