//
//  NSMutableData_Extensions.m
//  SQLKit
//
//  Created by Richard Penwell on 2/3/10.
//  Copyright 2010 Richard Penwell. All rights reserved.
//

#import "NSMutableData+Extensions.h"

@implementation NSIndexedLengthStringEntry

@synthesize position, data;

@end


@implementation NSMutableData (Extensions)

- (void)appendByte:(UInt8)byte
{
	[self appendBytes:(const void *)&byte length:sizeof(UInt8)];
}

- (void)appendUShort:(UInt16)aShort
{
	[self appendBytes:(const void *)&aShort length:sizeof(UInt16)];
}

- (void)appendUInt:(UInt32)aInt
{
	[self appendBytes:(const void *)&aInt length:sizeof(UInt32)];
}

- (void)appendUShort:(UInt16)aShort byteOrder:(CFByteOrder)order
{
	if (order == CFByteOrderBigEndian)
		[self appendUShort:CFSwapInt16HostToBig(aShort)];
	else
		[self appendUShort:CFSwapInt16HostToLittle(aShort)];
}

- (void)appendUInt:(UInt32)aInt byteOrder:(CFByteOrder)order
{
	if (order == CFByteOrderBigEndian)
		[self appendUInt:CFSwapInt32HostToBig(aInt)];
	else
		[self appendUInt:CFSwapInt32HostToLittle(aInt)];
}

- (void)appendUShortIndexedLengthString:(NSString*)string withDataSegment:(NSMutableArray*)data
{
	NSIndexedLengthStringEntry* entry = [NSIndexedLengthStringEntry new];
	[entry setPosition:[self length]];
	
	if (string != nil)
		[entry setData:[NSData dataWithBytes:[string cStringUsingEncoding:NSUnicodeStringEncoding] length:([string length] * 2)]];
	
	// Use an index of 0 as we dont know the true index yet
	[self appendUShort:0];
	if ([entry data] != nil)
		[self appendUShort:[string length]];
	else
		[self appendUShort:0];
	
	[data addObject:entry];
	[entry release];
}

- (void)appendUShortIndexedLengthData:(NSData*)dataObject withDataSegment:(NSMutableArray*)data
{
	NSIndexedLengthStringEntry* entry = [NSIndexedLengthStringEntry new];
	[entry setPosition:[self length]];
	
	if (data != nil)
		[entry setData:dataObject];
	
	// Use an index of 0 as we dont know the true index yet
	[self appendUShort:0];
	if ([entry data] != nil)
		[self appendUShort:([dataObject length]/2)];
	else
		[self appendUShort:0];
	
	[data addObject:entry];
	[entry release];
}

- (void)writeDataSegment:(NSArray*)data
{
	for (NSIndexedLengthStringEntry* entry in data)
	{
		NSUInteger index = [self length];
		[self setUShort:index atIndex:entry.position];
		if (entry.data != nil)
			[self appendData:entry.data];
	}
}

- (void)setUInt:(UInt32)aInt atIndex:(NSUInteger)index
{
	[self replaceBytesInRange:(NSRange){index, sizeof(UInt32)} withBytes:&aInt];
}

- (void)setUShort:(UInt16)aShort atIndex:(NSUInteger)index
{
	[self replaceBytesInRange:(NSRange){index, sizeof(UInt16)} withBytes:&aShort];
}

@end
