//
//  NSProcessInfo+MacAddress.m
//  SQLKit
//
//  Created by Richard Penwell on 2/8/10.
//  Copyright 2010 Richard Penwell. All rights reserved.
//

#import "NSProcessInfo+MacAddress.h"
#import <IOKit/IOKitLib.h>
#import <IOKit/network/IOEthernetInterface.h>
#import <IOKit/network/IONetworkInterface.h>
#import <IOKit/network/IOEthernetController.h>

static kern_return_t FindEthernetInterfaces(io_iterator_t *matchingServices)
{
	kern_return_t kernelResult = KERN_SUCCESS;
	CFMutableDictionaryRef matching;
	CFMutableDictionaryRef propertyMatching;
	
	matching = IOServiceMatching(kIOEthernetInterfaceClass);
	propertyMatching = CFDictionaryCreateMutable(kCFAllocatorDefault, 0, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
	CFDictionarySetValue(propertyMatching, CFSTR(kIOPrimaryInterface), kCFBooleanTrue);
	CFDictionarySetValue(matching, CFSTR(kIOPropertyMatchKey), propertyMatching);
	
	kernelResult = IOServiceGetMatchingServices(kIOMasterPortDefault, matching, matchingServices);
	CFRelease(propertyMatching);
	return kernelResult;
}

CFDataRef GetMacAddress(io_iterator_t matchingService)
{
	io_object_t objet = IOIteratorNext(matchingService);
	io_object_t controllerService;
	
	IORegistryEntryGetParentEntry(objet, kIOServicePlane, &controllerService);
	return (CFDataRef)IORegistryEntryCreateCFProperty(controllerService, CFSTR(kIOMACAddress), kCFAllocatorDefault, 0);
}

@implementation NSProcessInfo (MacAddress)

- (NSData*)macAddress
{
	io_iterator_t interfaceIterator;
	
	FindEthernetInterfaces(&interfaceIterator);

	
	return (NSData*)GetMacAddress(interfaceIterator);
}

@end
