//
//  TDSConnection.h
//  SQLKit
//
//  Created by Richard Penwell on 2/1/10.
//  Copyright 2010 Richard Penwell. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "TDSCommon.h"
#import "TDSConnectionDelegate.h"
#import "TDSCore.h"
#import "TDSResultPacket.h"
#import "TDSResultPacketDelegate.h"


extern const NSInteger kTDSDefaultPort;
extern const NSTimeInterval kTDSDefaultConnectionTimeout;
extern const NSInteger kTDSMajorVersion;
extern const NSInteger kTDSMinorVersion;

enum __TDSConnectionState {
	TDSConnectionStateNotOpen,
	TDSConnectionStateOpening,
	TDSConnectionStatePreloginSent,
	TDSConnectionStateLoginSent,
	TDSConnectionStateOpen
};
typedef enum __TDSConnectionState TDSConnectionState;

@interface TDSConnection : NSObject <NSStreamDelegate, TDSResultPacketDelegate> {
	NSInputStream* inputStream;
	NSOutputStream* outputStream;
	NSInteger port;
	NSHost* host;
	UInt8 packetNumber;
	NSString* applicationName;
	NSString* serverName;
	NSString* instanceName;
	NSString* databaseName;
	NSString* username;
	NSString* password;
	NSObject<TDSConnectionDelegate>* delegate;
	TDSCore* core;
	NSData* partialPacket;
	NSMutableArray* pendingPackets;
	TDSConnectionState state;
	UInt32 tdsServerVersion;
	UInt16 tdsServerBuild;
	TDSResultPacket* resultPacket;
}

+ (TDSConnection *)connectionWithHost:(NSHost *)host;
+ (TDSConnection *)connectionWithHost:(NSHost *)host andPort:(NSInteger)port;

- (TDSConnection *)initWithHost:(NSHost *)_host;
- (TDSConnection *)initWithHost:(NSHost *)_host andPort:(NSInteger)_port;

- (void)open;
- (void)openWithTimeout:(NSTimeInterval)timeout;
- (void)close;
- (NSStreamStatus)streamStatus;

- (void)writePacketData:(NSData *)data withTokenType:(TDSPacketType)type status:(TDSStatus)status;

@property (nonatomic) NSInteger port;
@property (nonatomic, retain) NSHost* host;
@property (nonatomic, retain) NSString* instanceName;
@property (nonatomic, retain) NSString* databaseName;
@property (nonatomic, retain) NSString* username;
@property (nonatomic, retain) NSString* password;
@property (nonatomic, retain) NSObject<TDSConnectionDelegate>* delegate;

@end
