//
//  TDSCore.m
//  SQLKit
//
//  Created by Richard Penwell on 2/2/10.
//  Copyright 2010 Richard Penwell. All rights reserved.
//

#import "TDSCore.h"
#import "TDSConnection.h"
#import "TDSLoginPacket.h"

const UInt32 kTDSVersion = 0x90000000;
const UInt16 kTDSBuild = 0x0000;

NSString* kTDSServerVersion = @"TDSServerVersion";
NSString* kTDSServerBuild = @"TDSServerBuild";
NSString* kTDSEncryptionType = @"TDSEncryptionType";
NSString* kTDSInstanceName = @"TDSInstanceName";
NSString* kTDSInstanceNameCorrect = @"TDSInstanceNameCorrect";
NSString* kTDSThreadId = @"TDSThreadId";
NSString* kTDSMultipleActiveResultSet = @"TDSMultipleActiveResultSet";

@implementation TDSCore

@synthesize connection;

- (void)preLoginWithParameters:(NSDictionary*)parameters;
{
	NSInteger instanceLength = ((connection.instanceName == nil) ? 1 : (connection.instanceName.length * 2 + 1));
	NSInteger tokensLength = (5 * 5) + 1;
	NSInteger location = tokensLength;
	NSMutableData* packetData = [NSMutableData dataWithCapacity:(tokensLength + 8 + instanceLength)];
	
	[packetData appendByte:TDSPreloginTokenTypeVersion];
	[packetData appendUShort:location byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:(sizeof(UInt32) + sizeof(UInt16)) byteOrder:CFByteOrderBigEndian];
	location += sizeof(UInt32) + sizeof(UInt16);
	
	[packetData appendByte:TDSPreloginTokenTypeEncryption];
	[packetData appendUShort:location byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:sizeof(UInt8) byteOrder:CFByteOrderBigEndian];
	location += sizeof(UInt8);
	
	[packetData appendByte:TDSPreloginTokenTypeInstance];
	[packetData appendUShort:location byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:instanceLength byteOrder:CFByteOrderBigEndian];
	location += instanceLength;
	
	[packetData appendByte:TDSPreloginTokenTypeThreadId];
	[packetData appendUShort:location byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:sizeof(UInt32) byteOrder:CFByteOrderBigEndian];
	location += sizeof(UInt32);
	
	[packetData appendByte:TDSPreloginTokenTypeMutipleActiveResultSet];
	[packetData appendUShort:location byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:sizeof(UInt8) byteOrder:CFByteOrderBigEndian];
	
	[packetData appendByte:TDSPreloginTokenTypeTerminator];
	
	[packetData appendUInt:kTDSVersion byteOrder:CFByteOrderBigEndian];
	[packetData appendUShort:kTDSBuild byteOrder:CFByteOrderBigEndian];
	[packetData appendByte:TDSEncryptionNotAvailable];
	if (connection.instanceName != nil)
	{
		[packetData appendBytes:[connection.instanceName cStringUsingEncoding:NSUnicodeStringEncoding] length:instanceLength];
	}
	else {
		[packetData appendByte:0];
	}
	
	[packetData appendUInt:(UInt32)connection];
	[packetData appendByte:TDSMultipleActiveResultsSetDisabled];

	[connection writePacketData:packetData withTokenType:TDSPacketTypePreLogin status:TDSStatusEndOfMessage];
}

- (NSDictionary*)processPreloginResponse:(NSData*)packetData
{
	NSDataReader* reader = [NSDataReader readerWithData:packetData];
	NSMutableDictionary* result = [NSMutableDictionary new];
	while ([reader hasMoreData])
	{
		Byte tokenName = [reader readByte];
		if (tokenName == TDSPreloginTokenTypeTerminator) break;
		UInt16 location = [reader readUShortWithByteOrder:CFByteOrderBigEndian];
		UInt16 size = [reader readUShortWithByteOrder:CFByteOrderBigEndian];
		NSInteger previousPosition = reader.position;
		reader.position = location;
		
		switch (tokenName) {
			case TDSPreloginTokenTypeVersion:
				if (size == 6)
				{
					[result setObject:[NSNumber numberWithInt:[reader readUIntWithByteOrder:CFByteOrderBigEndian]] forKey:kTDSServerVersion];
					[result setObject:[NSNumber numberWithShort:[reader readUShortWithByteOrder:CFByteOrderBigEndian]] forKey:kTDSServerBuild];
				}
				break;
			case TDSPreloginTokenTypeEncryption:
				if (size == 1) [result setObject:[NSNumber numberWithChar:[reader readByte]] forKey:kTDSEncryptionType];
				break;
			case TDSPreloginTokenTypeInstance:
				if (size == 1) [result setObject:[NSNumber numberWithBool:[reader readByte]] forKey:kTDSInstanceNameCorrect];
				break;
			case TDSPreloginTokenTypeThreadId:
				if (size == 4) [result setObject:[NSNumber numberWithInt:[reader readUIntWithByteOrder:CFByteOrderBigEndian]] forKey:kTDSThreadId];
				break;
			case TDSPreloginTokenTypeMutipleActiveResultSet:
				if (size == 1) [result setObject:[NSNumber numberWithBool:(BOOL)[reader readByte]] forKey:kTDSMultipleActiveResultSet];
				break;
			default:
				break;
		}
		
		reader.position = previousPosition;
	}
	
	return [result autorelease];
}

- (void)loginToDatabaseName:(NSString *)database username:(NSString *)username password:(NSString *)password
{
	TDSLoginPacket *loginPacket = [TDSLoginPacket new];
	
	loginPacket.serverName = [[connection host] name];
	loginPacket.databaseName = database;
	loginPacket.username = username;
	loginPacket.password = password;
	loginPacket.applicationName = @"SQLKit Client Application";
	loginPacket.clientVersion = 0x00101010;

	[connection writePacketData:[loginPacket createPacket] withTokenType:TDSPacketTypeLogin status:TDSStatusEndOfMessage];
	[loginPacket release];
}



@end
